/*
 * IV Mistrzostwa Wielkopolski w Programowaniu Zespolowym
 * Poznan, 1-2.XII.2005
 *
 * Program wzorcowy do zadania E - Powiartowany placek
 *
 * Autor programu: Piotr Gawron
 */

#include <algorithm>
#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
using namespace std;

// Show unifications
//#define DEBUG1

// Show points for every line
//#define DEBUG2

#define MAX_N 1004
#define MAX_P (MAX_N*(MAX_N-1)/2)
#define MAX_E (MAX_N*MAX_N)

typedef double real;

const real zero = 1e-10;
const real eps = 1e-9;

struct point;

struct edge {
	point * p[2];
	edge * next[2];
	bool visited[2];
};

struct point {
	real x;
	real y;
	bool used;

	vector<edge *> edges;
};

struct line {
	real a;
	real b;
	real c;

	vector<int> points;
};

line L[MAX_N];
edge E[MAX_E];
point P[MAX_P];
int R[MAX_P];
int n, p, e;
real w, h;

bool cmp1(int a, int b);
bool cmp2(edge * p, edge * q);
real angle(real x, real y);
void add(real x1, real y1, real x2, real y2);
real visit(bool direction, edge * e);
void clean_vertices();
void make_edges();
int get_point(int x);

int main()
{
	int t;

#if defined DEBUG2 || defined DEBUG2
	cerr.setf(ios::fixed);
	cerr.precision(4);
#endif

	scanf("%d", &t);
	
	n = 0;

	for (int tc = 0; tc < t; tc++) {
		real x1, x2, y1, y2;
		int np;
#if defined DEBUG2 || defined DEBUG2
		cerr <<"Test case " <<tc <<"\n";
#endif

		scanf("%lf%lf", &w, &h);
		scanf("%d", &np);

		for (int i = 0; i < n; i++)
			L[i].points.clear();

		n = 0;
		p = 0;
		e = 0;

		add(0, 0, w, 0); n++;
		add(w, 0, w, h); n++;
		add(w, h, 0, h); n++;
		add(0, h, 0, 0); n++;
	
		for (; n < np + 4; n++) {
			scanf("%lf%lf%lf%lf", &x1, &y1, &x2, &y2);
			add(x1, y1, x2, y2);
		}

		clean_vertices();
		make_edges();
		
		vector<real> area;

		for (int i = 0; i < e; i++) {
			if (!E[i].visited[0])
				area.push_back(fabs(visit(false, &E[i]))/2);

			if (!E[i].visited[1])
				area.push_back(fabs(visit(true, &E[i]))/2);
		}
		
		sort(area.begin(), area.end());

#ifdef DEBUG2
		for (int i = 0; i < n; i++) {
			cerr <<"Line " <<i + 1 <<"\n";
			for (int j = 0; j < L[i].points.size(); i++)
				cerr <<"[" <<get_point(L[i].points[j]) <<" (" <<P[get_point(L[i].points[j])].x <<", " <<P[get_point(L[i].points[j])].y <<") ";
			cerr <<"\n";
		}
#endif
	
		printf("%d\n", area.size() - 1);
		for (int i = 0; i < np; i++)
			printf("%.2lf ", area[area.size() - i - 2]);
		printf("\n");
		/*printf("%d\n", area.size() - 1);
		for (int i = 0; i < area.size() - 1; i++)
			printf("%.2llf\n", area[area.size() - i - 2]);
		printf("\n");*/
	}

	return 0;
}

bool cmp1(int a, int b)
{
	point * p1 = &P[get_point(a)];
	point * p2 = &P[get_point(b)];

	if (fabs(p1->x - p2->x) <= zero)
		return p1->y < p2->y;
	return p1->x < p2->x;
}

bool cmp2(edge * p, edge * q)
{
	point * a, * b, * ref;

	if (p->p[0] == q->p[0] || p->p[0] == q->p[1])
		ref = p->p[0];
	else
		ref = p->p[1];

	a = p->p[0] == ref ? p->p[1] : p->p[0];
	b = q->p[0] == ref ? q->p[1] : q->p[0];

	real a1 = angle(a->x - ref->x, a->y - ref->y);
	real a2 = angle(b->x - ref->x, b->y - ref->y);

	return a1 < a2;
}

real angle(real x, real y)
{	real d = fabs(x) + fabs(y);

	if (x >= 0 && y >= 0)
		return y / d;
	else if (x <= 0)
		return 2 - y / d;
	else
		return 4 + y / d;
}

int get_point(int x) {
	if (R[x] == x)
		return x;
	return R[x] = get_point(R[x]);
}

void add(real x1, real y1, real x2, real y2)
{
	real a, b;

	if (fabs(x1 - x2) > zero) {
		a = (y2 - y1) / (x2 - x1);
		b = y1 - a*x1;
		
		L[n].a = -a;
		L[n].b = 1;
		L[n].c = -b;
	} else {
		a = (x2 - x1) / (y2 - y1);
		b = x1 - a*y1;
		
		L[n].a = 1;
		L[n].b = -a;
		L[n].c = -b;
	}
	
	static int cc = 0;
	for (int j = 0; j < n; j++) {
		real ww = L[j].a*L[n].b - L[j].b*L[n].a;
		real wx = -L[j].c*L[n].b + L[j].b*L[n].c;
		real wy = -L[j].a*L[n].c + L[j].c*L[n].a;

		if (fabs(ww) > zero) {
			real x = wx / ww;
			real y = wy / ww;

			if (-zero <= x && x <= w + zero && -zero <= y && y <= h + zero) {
				P[p].x = x;
				P[p].y = y;
				P[p].used = true;
				P[p].edges.clear();

				R[p] = p;
				
				L[n].points.push_back(p);
				L[j].points.push_back(p);
				
				p++;
			}
		}
	}
}

real visit(bool direction, edge * e)
{
	edge * n;
	real c;

	if (e->visited[direction])
		return 0;

	e->visited[direction] = true;
	n = e->next[direction];
	c = e->p[!direction]->x*e->p[direction]->y - e->p[!direction]->y*e->p[direction]->x;
	
	if (e->p[direction] == n->p[direction])
		direction = !direction;

	return c + visit(direction, n);
}

void clean_vertices()
{
	for (int i = 0; i < n; i++) {
		sort(L[i].points.begin(), L[i].points.end(), cmp1);
		int k = 0;

		for (int j = 1; j < L[i].points.size(); j++) {
			int pa = get_point(L[i].points[j-1]);
			int pb = get_point(L[i].points[j]);

			if (pa != pb) {
				real x1 = P[pa].x;
				real y1 = P[pa].y;
				real x2 = P[pb].x;
				real y2 = P[pb].y;
				
				if ((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) < eps) {
#ifdef DEBUG1
					cerr <<"unif: (" <<x1 <<", " <<y1 <<") (" <<x2 <<", " <<y2 <<") " <<pa <<"\n";
#endif
					P[pb].used = false;
					R[pb] = pa;
				} else
					k++;
			} 
			L[i].points[k] = L[i].points[j];
		}
		L[i].points.erase(L[i].points.begin() + k + 1, L[i].points.end());
	}
}

void make_edges()
{
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < L[i].points.size(); j++) {
			if (j != 0) {
				E[e].p[0] = &P[get_point(L[i].points[j-1])];
				E[e].p[1] = &P[get_point(L[i].points[j])];

				E[e].next[0] = NULL;
				E[e].next[1] = NULL;

				E[e].visited[0] = false;
				E[e].visited[1] = false;

				E[e].p[0]->edges.push_back(&E[e]);
				E[e].p[1]->edges.push_back(&E[e]);
				
				e++;
			}
		}
	}

	for (int i = 0; i < p; i++) {
		if (P[i].used) {
			sort(P[i].edges.begin(), P[i].edges.end(), cmp2);

			for (int j = 0; j < P[i].edges.size(); j++) {
				if (&P[i] == P[i].edges[j]->p[0])
					P[i].edges[j]->next[0] = P[i].edges[ (j + 1) % P[i].edges.size() ];
				else
					P[i].edges[j]->next[1] = P[i].edges[ (j + 1) % P[i].edges.size() ];
			}

		}
	}
}

